# noinspection SpellCheckingInspection
palette = \
    {
        "minecraft:air": [0, 0, 0],
        # "minecraft:acacia_log": [97, 97, 87],
        # "minecraft:acacia_planks": [90, 90, 50],
        "minecraft:ancient_debris": [96, 64, 56],
        "minecraft:andesite": [136, 136, 137],
        "minecraft:barrel": [122, 90, 52],
        "minecraft:basalt": [73, 73, 78],
        "minecraft:bedrock": [85, 85, 85],
        "minecraft:bee_nest": [161, 127, 88],
        "minecraft:birch_log": [217, 215, 210],
        "minecraft:birch_planks": [192, 175, 121],
        "minecraft:black_concrete": [8, 10, 15],
        "minecraft:black_concrete_powder": [25, 27, 32],
        "minecraft:black_glazed_terracotta": [68, 30, 32],
        "minecraft:black_terracotta": [37, 23, 16],
        "minecraft:black_stained_glass": [25, 25, 25],
        "minecraft:black_wool": [21, 21, 26],
        "minecraft:blackstone": [42, 35, 41],
        "minecraft:blast_furnace": [108, 108, 107],
        "minecraft:blue_concrete": [45, 47, 143],
        "minecraft:blue_concrete_powder": [70, 73, 167],
        "minecraft:blue_glazed_terracotta": [47, 65, 139],
        "minecraft:blue_ice": [116, 168, 253],
        "minecraft:blue_terracotta": [74, 60, 91],
        "minecraft:blue_stained_glass": [51, 76, 178],
        "minecraft:blue_wool": [53, 57, 157],
        "minecraft:bone_block": [229, 226, 208],
        "minecraft:bone_meal": [159, 158, 180],
        "minecraft:bookshelf": [117, 95, 60],
        "minecraft:brain_coral": [198, 85, 152],
        "minecraft:brain_coral_block": [207, 91, 159],
        "minecraft:bricks": [151, 98, 83],
        "minecraft:brown_concrete": [96, 60, 32],
        "minecraft:brown_concrete_powder": [126, 85, 54],
        "minecraft:brown_glazed_terracotta": [120, 106, 86],
        "minecraft:brown_stained_glass": [102, 76, 51],
        "minecraft:brown_mushroom_block": [149, 112, 81],
        "minecraft:brown_terracotta": [77, 51, 36],
        "minecraft:brown_wool": [114, 72, 41],
        "minecraft:bubble_coral": [161, 24, 160],
        "minecraft:bubble_coral_block": [165, 26, 162],
        "minecraft:cartography_table": [72, 50, 34],
        "minecraft:carved_pumpkin": [150, 84, 17],
        "minecraft:chiseled_nether_bricks": [47, 24, 28],
        "minecraft:chiseled_polished_blackstone": [54, 49, 57],
        "minecraft:chiseled_quartz_block": [232, 227, 218],
        "minecraft:chiseled_red_sandstone": [183, 97, 28],
        "minecraft:chiseled_sandstone": [216, 203, 155],
        "minecraft:chiseled_stone_bricks": [120, 119, 120],
        "minecraft:coal_block": [16, 16, 16],
        "minecraft:coal_ore": [116, 116, 116],
        "minecraft:coarse_dirt": [119, 86, 59],
        "minecraft:cobblestone": [128, 127, 128],
        "minecraft:cracked_nether_bricks": [40, 20, 24],
        "minecraft:cracked_polished_blackstone_bricks": [44, 38, 44],
        "minecraft:cracked_stone_bricks": [118, 118, 118],
        "minecraft:crimson_nylium": [131, 31, 31],
        "minecraft:crimson_planks": [101, 49, 71],
        "minecraft:crimson_stem": [93, 26, 30],
        "minecraft:crying_obsidian": [33, 10, 60],
        "minecraft:cut_red_sandstone": [189, 102, 32],
        "minecraft:cut_sandstone": [218, 206, 160],
        "minecraft:cyan_concrete": [21, 119, 136],
        "minecraft:cyan_concrete_powder": [37, 148, 157],
        "minecraft:cyan_glazed_terracotta": [52, 119, 125],
        "minecraft:cyan_stained_glass": [76, 127, 153],
        "minecraft:cyan_terracotta": [87, 91, 91],
        "minecraft:cyan_wool": [21, 138, 145],
        "minecraft:dark_oak_log": [60, 47, 26],
        "minecraft:dark_oak_planks": [67, 43, 20],
        "minecraft:dark_prismarine": [52, 92, 76],
        "minecraft:dead_brain_coral_block": [124, 118, 114],
        "minecraft:dead_bubble_coral_block": [132, 124, 119],
        "minecraft:dead_fire_coral_block": [132, 124, 120],
        "minecraft:dead_horn_coral_block": [134, 126, 122],
        "minecraft:dead_tube_coral_block": [130, 123, 120],
        "minecraft:diorite": [189, 188, 189],
        "minecraft:dirt": [134, 96, 67],
        "minecraft:dried_kelp_block": [38, 49, 30],
        "minecraft:emerald_block": [42, 203, 88],
        "minecraft:emerald_ore": [117, 137, 124],
        "minecraft:end_stone": [220, 223, 158],
        "minecraft:end_stone_bricks": [218, 224, 162],
        "minecraft:fire_coral_block": [164, 35, 47],
        "minecraft:fletching_table": [173, 155, 111],
        "minecraft:gilded_blackstone": [57, 44, 39],
        "minecraft:glass": [176, 214, 219],
        "minecraft:glowstone": [172, 131, 84],
        "minecraft:gold_block": [246, 208, 62],
        "minecraft:gold_ore": [144, 140, 125],
        "minecraft:granite": [149, 103, 86],
        "minecraft:grass_block": [127, 107, 66],
        "minecraft:grass_path": [142, 106, 70],
        "minecraft:gravel": [132, 127, 127],
        "minecraft:gray_concrete": [55, 58, 62],
        "minecraft:gray_concrete_powder": [77, 81, 85],
        "minecraft:gray_glazed_terracotta": [83, 90, 94],
        "minecraft:gray_stained_glass": [76, 76, 76],
        "minecraft:gray_terracotta": [58, 42, 36],
        "minecraft:gray_wool": [63, 68, 72],
        "minecraft:green_concrete": [73, 91, 36],
        "minecraft:green_concrete_powder": [97, 119, 45],
        "minecraft:green_glazed_terracotta": [117, 142, 67],
        "minecraft:green_stained_glass": [102, 127, 51],
        "minecraft:green_terracotta": [76, 83, 42],
        "minecraft:green_wool": [85, 110, 28],
        "minecraft:hay_block": [166, 136, 38],
        "minecraft:honey_block": [241, 146, 18],
        "minecraft:honeycomb_block": [229, 148, 30],
        "minecraft:horn_coral_block": [216, 200, 66],
        "minecraft:ice": [146, 184, 254],
        "minecraft:iron_block": [220, 220, 220],
        "minecraft:iron_ore": [136, 131, 127],
        "minecraft:jack_o_lantern": [215, 152, 53],
        "minecraft:jukebox": [4, 5, 5],
        "minecraft:jungle_log": [85, 68, 25],
        "minecraft:jungle_planks": [160, 115, 81],
        "minecraft:lapis_block": [31, 67, 140],
        "minecraft:lapis_ore": [99, 111, 133],
        "minecraft:light_blue_concrete": [36, 137, 199],
        "minecraft:light_blue_concrete_powder": [74, 181, 213],
        "minecraft:light_blue_glazed_terracotta": [95, 165, 209],
        "minecraft:light_blue_stained_glass": [102, 153, 216],
        "minecraft:light_blue_terracotta": [113, 109, 138],
        "minecraft:light_blue_wool": [58, 175, 217],
        "minecraft:light_gray_concrete": [125, 125, 115],
        "minecraft:light_gray_concrete_powder": [155, 155, 148],
        "minecraft:light_gray_glazed_terracotta": [144, 166, 168],
        "minecraft:light_gray_stained_glass": [153, 153, 153],
        "minecraft:light_gray_terracotta": [135, 107, 98],
        "minecraft:light_gray_wool": [142, 142, 135],
        "minecraft:lime_concrete": [94, 169, 24],
        "minecraft:lime_concrete_powder": [125, 189, 42],
        "minecraft:lime_glazed_terracotta": [163, 198, 55],
        "minecraft:lime_stained_glass": [127, 204, 25],
        "minecraft:lime_terracotta": [104, 118, 53],
        "minecraft:lime_wool": [112, 185, 26],
        "minecraft:lodestone": [119, 120, 123],
        "minecraft:loom": [76, 60, 36],
        "minecraft:magenta_concrete": [169, 48, 159],
        "minecraft:magenta_concrete_powder": [193, 84, 185],
        "minecraft:magenta_glazed_terracotta": [208, 100, 192],
        "minecraft:magenta_stained_glass": [178, 76, 216],
        "minecraft:magenta_terracotta": [150, 88, 109],
        "minecraft:magenta_wool": [190, 69, 180],
        "minecraft:magma_block": [143, 63, 32],
        "minecraft:mossy_cobblestone": [110, 118, 95],
        "minecraft:mossy_stone_bricks": [115, 121, 105],
        "minecraft:mushroom_stem": [203, 197, 186],
        "minecraft:mycelium": [113, 87, 71],
        "minecraft:nether_brick": [3, 5, 4],
        "minecraft:nether_bricks": [3, 5, 4],
        "minecraft:nether_gold_ore": [115, 55, 42],
        "minecraft:nether_quartz_ore": [118, 66, 62],
        "minecraft:nether_wart_block": [115, 3, 2],
        "minecraft:netherite_block": [67, 61, 64],
        "minecraft:netherrack": [98, 38, 38],
        "minecraft:oak_log": [109, 85, 51],
        "minecraft:oak_planks": [162, 131, 79],
        "minecraft:obsidian": [15, 11, 25],
        "minecraft:orange_concrete": [224, 97, 1],
        "minecraft:orange_concrete_powder": [227, 132, 32],
        "minecraft:orange_glazed_terracotta": [155, 147, 92],
        "minecraft:orange_stained_glass": [216, 127, 51],
        "minecraft:orange_terracotta": [162, 84, 38],
        "minecraft:orange_wool": [241, 118, 20],
        "minecraft:packed_ice": [142, 180, 250],
        "minecraft:pink_concrete": [214, 101, 143],
        "minecraft:pink_concrete_powder": [229, 153, 181],
        "minecraft:pink_glazed_terracotta": [235, 155, 182],
        "minecraft:pink_stained_glass": [242, 127, 165],
        "minecraft:pink_terracotta": [162, 78, 79],
        "minecraft:pink_wool": [238, 141, 172],
        "minecraft:podzol": [123, 88, 57],
        "minecraft:polished_andesite": [132, 135, 134],
        "minecraft:polished_basalt": [89, 88, 92],
        "minecraft:polished_blackstone": [53, 49, 57],
        "minecraft:polished_blackstone_bricks": [47, 41, 48],
        "minecraft:polished_diorite": [193, 193, 195],
        "minecraft:polished_granite": [154, 107, 89],
        "minecraft:prismarine": [99, 156, 151],
        "minecraft:prismarine_bricks": [99, 172, 158],
        "minecraft:pumpkin": [196, 115, 24],
        "minecraft:purple_concrete": [100, 32, 156],
        "minecraft:purple_concrete_powder": [132, 56, 178],
        "minecraft:purple_glazed_terracotta": [110, 48, 152],
        "minecraft:purple_stained_glass": [127, 63, 178],
        "minecraft:purple_terracotta": [118, 70, 86],
        "minecraft:purple_wool": [122, 42, 173],
        "minecraft:purpur_block": [170, 126, 170],
        "minecraft:purpur_pillar": [172, 130, 172],
        "minecraft:quartz_block": [237, 230, 224],
        "minecraft:quartz_bricks": [235, 229, 222],
        "minecraft:quartz_pillar": [236, 231, 224],
        "minecraft:red_concrete": [142, 33, 33],
        "minecraft:red_concrete_powder": [168, 54, 51],
        "minecraft:red_glazed_terracotta": [182, 60, 53],
        "minecraft:red_stained_glass": [153, 51, 51],
        "minecraft:red_mushroom": [217, 75, 68],
        "minecraft:red_mushroom_block": [200, 47, 45],
        "minecraft:red_nether_bricks": [70, 7, 9],
        "minecraft:red_sand": [191, 103, 33],
        "minecraft:red_sandstone": [187, 99, 29],
        "minecraft:red_terracotta": [143, 61, 47],
        "minecraft:red_wool": [161, 39, 35],
        "minecraft:redstone_block": [4, 3, 4],
        "minecraft:redstone_lamp": [95, 55, 30],
        "minecraft:redstone_ore": [133, 108, 108],
        "minecraft:sand": [219, 207, 163],
        "minecraft:sandstone": [216, 203, 156],
        "minecraft:sea_lantern": [172, 200, 190],
        "minecraft:shroomlight": [241, 147, 71],
        "minecraft:slime_block": [2, 2, 2],
        "minecraft:smithing_table": [64, 28, 24],
        "minecraft:smoker": [107, 106, 104],
        "minecraft:smooth_stone": [159, 159, 159],
        "minecraft:snow_block": [249, 254, 254],
        "minecraft:soul_sand": [81, 62, 51],
        "minecraft:soul_soil": [76, 58, 47],
        "minecraft:spruce_log": [59, 38, 17],
        "minecraft:spruce_planks": [115, 85, 49],
        "minecraft:stone": [126, 126, 126],
        "minecraft:stone_bricks": [122, 122, 122],
        "minecraft:stripped_acacia_log": [175, 93, 60],
        "minecraft:stripped_birch_log": [197, 176, 118],
        "minecraft:stripped_crimson_stem": [137, 57, 90],
        "minecraft:stripped_dark_oak_log": [97, 76, 50],
        "minecraft:stripped_jungle_log": [171, 133, 85],
        "minecraft:stripped_oak_log": [177, 144, 86],
        "minecraft:stripped_spruce_log": [116, 90, 52],
        "minecraft:stripped_warped_stem": [58, 151, 148],
        "minecraft:target": [229, 176, 168],
        "minecraft:terracotta": [152, 94, 68],
        "minecraft:tube_coral_block": [49, 87, 207],
        "minecraft:warped_fungus": [74, 109, 88],
        "minecraft:warped_nylium": [43, 114, 101],
        "minecraft:warped_planks": [43, 105, 99],
        "minecraft:warped_wart_block": [23, 120, 121],
        "minecraft:wet_sponge": [171, 181, 70],
        "minecraft:white_concrete": [207, 213, 214],
        "minecraft:white_concrete_powder": [226, 227, 228],
        "minecraft:white_glazed_terracotta": [188, 212, 203],
        "minecraft:white_stained_glass": [255, 255, 255],
        "minecraft:white_terracotta": [210, 178, 161],
        "minecraft:white_wool": [234, 236, 237],
        "minecraft:yellow_concrete": [241, 175, 21],
        "minecraft:yellow_concrete_powder": [233, 199, 55],
        "minecraft:yellow_glazed_terracotta": [234, 192, 89],
        "minecraft:yellow_stained_glass": [229, 229, 51],
        "minecraft:yellow_terracotta": [186, 133, 35],
        "minecraft:yellow_wool": [249, 198, 40]
    }
